import gtk
import gobject
import calendar
import datetime
import os
import sys

"""
	
	QuitMeter Python application.
	Written by Robin Rawson-Tetley, 30 March, 2007
	Updated 3rd Feb, 2010 to add milestones.

	GTK application to display the amount of time
	since you stopped smoking, the number of cigs
	not smoked, along with the amount of money and 
	your own life saved.

	This application is covered by the terms of the
	GNU General Public Licence Version 2.0 or any 
	later version.

	Edit the values below to suit your smoking
	habits.
	
"""

# The date you stopped smoking - my first attempt (shown here) 
# was 9th January, 2004 at 11:09pm. Until I started again in
# May 2006.
#datestopped = datetime.datetime(2004, 1, 9, 23, 9, 0)
# New stop date!
#datestopped = datetime.datetime(2008, 1, 13, 22, 45, 0)
datestopped = datetime.datetime(2010, 01, 26, 0, 0, 0)
# Number of cigarettes you smoked per day when you were
# a filthy smoker...
cigsperday = 40
# The cost of a packet of 20 cigarettes in your local currency
twentycigs = 5.01
# Your currency
currency = "GBP"
# The amount of life you remove (in minutes) for every cig smoked
# (I know these figures are bollocks but they make you feel better!)
# Also, if you said it takes 5 minutes to smoke one, that's 5 minutes
# wasted and 5 minutes saved!
lifepercig = 5

# Benefits of stopping after X hours
benefits = ( ( 1, "1 hour: Blood pressure, pulse rate, extremity temperature return to normal"),
	( 8, "8 hours: Remaining nicotine in bloodstream has fallen to 6.25% of peak levels - a 93.25% reduction" ),
	( 12, "12 hours: Blood oxygen level increased to normal" ),
	( 24, "24 hours: Peak anxiety" ),
	( 48, "48 hours: Damaged nerve endings start to regrow, sense of smell and taste begin to return" ),
	( 72, "72 hours: Withdrawal symptoms have peaked. Breathing is easier." ),
	( 24 * 5, "5 days: Down to 3 craves per day" ),
	( 24 * 10, "10 days: Down to 2 craves per day" ),
	( 24 * 12, "12 days: Blood circulating in gums and teeth similar to that of a non-smoker" ),
	( 24 * 14, "2 weeks: Anger, anxiety, restlessness and depression have ended" ),
	( 24 * 21, "3 weeks: Brain acetylcholine receptor counts return to non-smoker levels" ),
	( 24 * 28, "4 weeks: Heart attack risk has started to reduce. Lung function is improving" ),
	( 24 * 35, "5 weeks: Circulation improved, walking is easier, smokers cough has disappeared" ),
	( 24 * 30 * 6, "6 months: Cilia have regrown in lungs, improving immune system and function" ),
	( 24 * 365, "1 year: Risk of coronary heart disease has dropped to less than half than that of a smoker" ),
	( 24 * 365 * 5, "5 years: Risk of stroke has declined to that of a non-smoker" ),
	( 24 * 365 * 10, "10 years: Risk of lung cancer has declined by almost half" ),
	( 24 * 365 * 13, "13 years: Risk of smoking induced tooth loss has declined to that of a never-smoker" ),
	( 24 * 365 * 15, "15 years: Risk of coronary heart disease is now that of a never-smoker") )

class QuitForm:

    animpos = 1

    def destroy(self, widget, data=None):
        gtk.main_quit()

    def delete_event(self, widget, event, data=None):
        return gtk.FALSE

    def statusactivate(self, widget=None, event=None, data=None):
    	if (self.window.is_active()):
		self.window.hide()
	else:
		self.window.show()

    def update(self):
    
    	# Update the labels to display the right information

	# The amount of time you stopped for
	now = datetime.datetime(2004, 1, 1)
	timediff = now.today() - datestopped	
	secs = timediff.seconds + (timediff.days * 24 * 60 * 60)
	seconds = secs
	days = secs / (60 * 60 * 24)
	secs -= (days * 24 * 60 * 60)	
  	hrs = secs / (60 * 60)
	secs -= (hrs * 60 * 60)
	mins = (secs / 60) % 60
	secs = secs % 60
	timestoppedstr = "Time stopped: %d days, %d hours, %d minutes and %d seconds" % ( days, hrs, mins, secs )
	self.lblTimeStopped.set_text(timestoppedstr)

	# Cigarettes not smoked
	hoursstopped = float(seconds) / 60 / 60
	cigsperhour = float(cigsperday) / 24
	cigsnotsmoked = round(hoursstopped * cigsperhour, 2)
	self.lblCigarettes.set_text("Cigarettes not smoked: " + str(cigsnotsmoked))
	
	# The amount of money saved
	saved = round((cigsnotsmoked / 20) * twentycigs, 2)
	self.lblMoneySaved.set_text("Money saved: " + "(" + currency + ") " + str(saved))

	# The amount of life saved
	lifesaved = round((cigsnotsmoked * lifepercig) / 60 / 24, 2)
	self.lblLifeSaved.set_text("Life saved: " + str(lifesaved) + " days")

	# Any milestone
	milestone = ""
	for m in benefits:
		if hoursstopped > m[0]:
			milestone = m[1]
		else:
			break
	self.lblMilestone.set_text(milestone)

	# Load a new animation image (if the window has the focus)
	if self.window.is_active():
		self.anim.set_from_file("anim" + str(self.animpos) + ".png")
		self.animpos += 1
		if self.animpos == 6: self.animpos = 1

	# Update the tray icon tooltip text 
	self.trayicon.set_tooltip("%s, \n%s unsmoked cigarettes, \n%s%s saved, \n%s days of life saved." % (timestoppedstr, str(cigsnotsmoked), str(currency), str(saved), str(lifesaved) ))

	# Keep setting the timer
	return True
	

    def __init__(self, quiet):
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.connect("delete_event", self.delete_event)
        self.window.connect("destroy", self.destroy)
        self.window.set_border_width(10)
	self.window.set_title("QuitMeter")
	self.layout = gtk.Fixed()

	# Time since we stopped
	self.lblTimeStopped = gtk.Label("Timestopped")
	self.layout.put(self.lblTimeStopped, 70, 10)

	# Life saved
	self.lblLifeSaved = gtk.Label("LifeSaved")
	self.layout.put(self.lblLifeSaved, 70, 30)

	# Money saved
	self.lblMoneySaved = gtk.Label("MoneySaved")
	self.layout.put(self.lblMoneySaved, 70, 50)

	# Cigarettes not smoked
	self.lblCigarettes = gtk.Label("CigsNotSmoked")
	self.layout.put(self.lblCigarettes, 70, 70)

	# Milestone
	self.lblMilestone = gtk.Label("Milestone")
	self.layout.put(self.lblMilestone, 70, 90)

	# Animation
	self.anim = gtk.Image()
	self.layout.put(self.anim, 10, 10)

	# Status Icon
	self.trayicon = gtk.StatusIcon()
	self.trayicon.set_from_file("quitmeter.png")
	self.trayicon.connect("activate", self.statusactivate)

	# Display the window
	self.layout.show_all()
	self.window.add(self.layout)
	if not quiet:
        	self.window.show()

    def main(self):
        # All PyGTK applications must have a gtk.main(). Control ends here
        # and waits for an event to occur (like a key press or mouse event).
	gtk.threads_init()
	gobject.timeout_add(200, self.update) # Poll every 1 second for update
        gtk.main()




if __name__ == "__main__":

	# Parse arguments
	quiet = False
	for a in sys.argv:
		if a == "--help" or a == "-?" or a == "-h":
			print "Usage: quitmeter [-q]"
			print ""
			print "    -q Quiet startup (minimise to task tray)"
			print ""
			sys.exit(0)
		if a == "-q":
			quiet = True

	quit = QuitForm(quiet)
	quit.update()
	quit.main()
